@@ -165,8 +165,6 @@ pool_start_query(POOL_QUERY_CONTEXT * query_context, char *query, int len, Node
 			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
 		query_context->skip_cache_commit = false;
-		query_context->atEnd = false;
-		query_context->partial_fetch = false;
 		session_context->query_context = query_context;
 		MemoryContextSwitchTo(old_context);
 	}
@@ -191,7 +189,7 @@ pool_set_node_to_be_sent(POOL_QUERY_CONTEXT * query_context, int node_id)
 }
 
 /*
- * Unspecify DB node to send query
+ * Unspecified DB node to send query
  */
 void
 pool_unset_node_to_be_sent(POOL_QUERY_CONTEXT * query_context, int node_id)
@@ -242,23 +240,20 @@ pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT * query_context)
 			{
 				/*
 				 * If load balance mode is disabled, only send to the primary node.
-				 * If primary node does not exist, send to the main node.
+				 * or send to the main node if primary node does not exist.
 				 */
 				if (!pool_config->load_balance_mode)
 				{
 					if (i == PRIMARY_NODE_ID ||
 						(PRIMARY_NODE_ID < 0 && MAIN_NODE_ID == i))
-					{
 						query_context->where_to_send[i] = true;
-						break;
-					}
-					continue;
+					break;
 				}
 				else
 					/*
 					 * If the node is not primary node nor load balance node,
-					 * there's no point to send query except statement level
-					 * load balance is enabled.
+					 * there's no point to send query except statement load
+					 * balance is enabled.
 					 */
 					if (!pool_config->statement_level_load_balance &&
 						i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
@@ -363,16 +358,7 @@ pool_virtual_main_db_node_id(void)
 	sc = pool_get_session_context(true);
 	if (!sc)
 	{
-		/*
-		 * We used to return REAL_MAIN_NODE_ID here.  Problem with it is, it
-		 * is possible that REAL_MAIN_NODE_ID could be changed
-		 * anytime. Suppose REAL_MAIN_NODE_ID == my_main_node_id == 1. Then
-		 * due to failback, REAL_MAIN_NODE_ID is changed to 0. Then
-		 * MAIN_CONNECTION(cp) will return NULL and any reference to it will
-		 * cause segmentation fault. To prevent the issue we should return
-		 * my_main_node_id instead.
-		 */
-		return my_main_node_id;
+		return REAL_MAIN_NODE_ID;
 	}
 
 	if (sc->in_progress && sc->query_context)
@@ -415,7 +401,7 @@ pool_virtual_main_db_node_id(void)
 	}
 
 	/*
-	 * No query context exists.  If in streaming replication mode, returns primary node
+	 * No query context exists.  If in native replication mode, returns primary node
 	 * if exists.  Otherwise returns my_main_node_id, which represents the
 	 * last REAL_MAIN_NODE_ID.
 	 */
@@ -458,7 +444,6 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 {
 	POOL_SESSION_CONTEXT *session_context;
 	POOL_CONNECTION_POOL *backend;
-	int			i;
 
 	CHECK_QUERY_CONTEXT_IS_VALID;
 
@@ -470,24 +455,6 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 	 */
 	pool_clear_node_to_be_sent(query_context);
 
-	/*
-	 * If there is "NO LOAD BALANCE" comment, we send only to main node.
-	 */
-	if (!strncasecmp(query, NO_LOAD_BALANCE, NO_LOAD_BALANCE_COMMENT_SZ))
-	{
-		pool_set_node_to_be_sent(query_context,
-								 MAIN_REPLICA ? PRIMARY_NODE_ID : REAL_MAIN_NODE_ID);
-		for (i = 0; i < NUM_BACKENDS; i++)
-		{
-			if (query_context->where_to_send[i])
-			{
-				query_context->virtual_main_node_id = i;
-				break;
-			}
-		}
-		return;
-	}
-
 	/*
 	 * In raw mode, we send only to main node. Simple enough.
 	 */
@@ -570,7 +537,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 				 */
 
 				ereport(DEBUG1,
-						(errmsg("checking load balance precondtions. TSTATE:%c wrting_trancation:%d failed_transaction:%d isolation:%d",
+						(errmsg("checking load balance preconditions. TSTATE:%c writing_transaction:%d failed_transaction:%d isolation:%d",
 								TSTATE(backend, PRIMARY_NODE_ID),
 								pool_is_writing_transaction(),
 								pool_is_failed_transaction(),
@@ -588,21 +555,6 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					 * Load balance if possible
 					 */
 
-					/*
-					 * If replication delay is too much, we prefer to send to
-					 * the primary.
-					 */
-					if (STREAM &&
-						pool_config->delay_threshold &&
-						bkinfo->standby_delay > pool_config->delay_threshold)
-					{
-						ereport(DEBUG1,
-								(errmsg("could not load balance because of too much replication delay"),
-								 errdetail("destination = %d for query= \"%s\"", dest, query)));
-
-						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
-					}
-
 					/*
 					 * If system catalog is used in the SELECT, we prefer to
 					 * send to the primary. Example: SELECT * FROM pg_class
@@ -613,7 +565,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					 * primary system catalog. Please note that this test must
 					 * be done *before* test using pool_has_temp_table.
 					 */
-					else if (pool_has_system_catalog(node))
+					if (pool_has_system_catalog(node))
 					{
 						ereport(DEBUG1,
 								(errmsg("could not load balance because systems catalogs are used"),
@@ -674,11 +626,44 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 					else
 					{
 						if (pool_config->statement_level_load_balance)
+						{
 							session_context->load_balance_node_id = select_load_balancing_node();
+							bkinfo = pool_get_node_info(session_context->load_balance_node_id);
+						}
 
-						session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
-						pool_set_node_to_be_sent(query_context,
-												 session_context->query_context->load_balance_node_id);
+						/*
+						 * As streaming replication delay is too much, if
+						 * prefer_lower_delay_standby is true then elect new
+						 * load balance node which is lowest delayed,
+						 * false then send to the primary.
+						 */
+						if (STREAM &&
+							pool_config->delay_threshold &&
+							bkinfo->standby_delay > pool_config->delay_threshold)
+						{
+							ereport(DEBUG1,
+									(errmsg("could not load balance because of too much replication delay"),
+									 errdetail("destination = %d for query= \"%s\"", dest, query)));
+
+							if (pool_config->prefer_lower_delay_standby)
+							{
+								int new_load_balancing_node = select_load_balancing_node();
+
+								session_context->load_balance_node_id = new_load_balancing_node;
+								session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
+								pool_set_node_to_be_sent(query_context, session_context->query_context->load_balance_node_id);
+							}
+							else
+							{
+								pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+							}
+						}
+						else
+						{
+							session_context->query_context->load_balance_node_id = session_context->load_balance_node_id;
+							pool_set_node_to_be_sent(query_context,
+													 session_context->query_context->load_balance_node_id);
+						}
 					}
 				}
 				else
@@ -918,7 +903,6 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		}
 
 		per_node_statement_log(backend, i, string);
-		per_node_statement_notice(backend, i, string);
 		stat_count_up(i, query_context->parse_tree);
 		send_simplequery_message(CONNECTION(backend, i), len, string, MAJOR(backend));
 	}
@@ -970,7 +954,7 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		 * that any session object (i.e. named statement) does not remain in
 		 * the last session.
 		 */
-		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend", true) == 'E')
+		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend: ", true) == 'E')
 			reset_query_error = true;
 	}
 
@@ -1103,7 +1087,6 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			}
 
 			per_node_statement_log(backend, i, msgbuf);
-			per_node_statement_notice(backend, i, msgbuf);
 		}
 
 		/* if Execute message, count up stats count */
@@ -1182,7 +1165,7 @@ pool_extended_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 			 * does not report what statement caused that error and make users
 			 * confused.
 			 */
-			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend", true);
+			per_node_error_log(backend, i, str, "pool_send_and_wait: Error or notice message from backend: ", true);
 		}
 	}
 
